package net.wanji.datacenter.kafka;

import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.datacenter.pojo.dto.ImbalanceAlgorithmDTO;
import net.wanji.datacenter.service.CrossStrategyOptService;
import org.apache.kafka.clients.consumer.Consumer;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.listener.KafkaListenerErrorHandler;
import org.springframework.kafka.listener.ListenerExecutionFailedException;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.messaging.Message;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @author duanruiming
 * @date 2024/11/23 15:44
 * @description 神思路口策略优化结果
 */
@Component
@Slf4j
public class CrossStrategyOptConsumerHandler implements KafkaListenerErrorHandler {

    @Resource
    private CrossStrategyOptService crossStrategyOptService;

    /**
     * 相位空放优化指标周期
     * @param record
     * @param acknowledgment
     * @throws Exception
     */
    @KafkaListener(topics = {"${kafka-consumer.phaseEmptyTopic}"}, groupId = "group")
    public void receivePhaseEmptyData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) throws Exception {
        try {
            String message = String.valueOf(record.value());
            if (StringUtils.isNotBlank(message)) {
                crossStrategyOptService.savePhaseEmpty(message);
            }
        } catch (Exception e) {
            log.error("相位空放phaseEmpty数据处理异常；", e);
            throw new Exception(e);
        }
        acknowledgment.acknowledge();
    }

    /**
     * 失衡算法结果
     *
     * @param record
     * @param acknowledgment
     * @throws Exception
     */
    @KafkaListener(topics = {"${kafka-consumer.imbalanceAlgorithmResultTopic}"}, groupId = "group")
    public void receiveImbalanceData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) throws Exception {
        String message = String.valueOf(record.value());
        if (StringUtils.isNotBlank(message)) {
            ObjectMapper mapper = JacksonUtils.getInstance();
            ImbalanceAlgorithmDTO imbalanceAlgorithmDTO = mapper.readValue(message, ImbalanceAlgorithmDTO.class);
            // 设置3,优化方案
            imbalanceAlgorithmDTO.setRtnType(3);
            crossStrategyOptService.save(imbalanceAlgorithmDTO);
        }
        acknowledgment.acknowledge();
    }

    @KafkaListener(topics = {"${kafka-consumer.overflowAlgorithmResultTopic}"}, groupId = "group")
    public void receiveOverFlowData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) throws Exception {
        String message = String.valueOf(record.value());
        if (StringUtils.isNotBlank(message)) {
            ObjectMapper mapper = JacksonUtils.getInstance();
            ImbalanceAlgorithmDTO imbalanceAlgorithmDTO = mapper.readValue(message, ImbalanceAlgorithmDTO.class);
            crossStrategyOptService.save(imbalanceAlgorithmDTO);
        }
        acknowledgment.acknowledge();
    }
    @Override
    @NonNull
    public Object handleError(Message<?> message, ListenerExecutionFailedException e) {
        return new Object();
    }

    @Override
    @NonNull
    public Object handleError(Message<?> message, ListenerExecutionFailedException exception, Consumer<?, ?> consumer) {
        String errorMessage = String.format("监听主题：%s，消费者详情：%s，异常信息：%s，消息详情：%s",
                consumer.listTopics(), consumer.groupMetadata(), exception, message);
        log.error(errorMessage);
        return KafkaListenerErrorHandler.super.handleError(message, exception, consumer);
    }

    public static void main(String[] args) throws Exception {
        ObjectMapper mapper = JacksonUtils.getInstance();
        ImbalanceAlgorithmDTO imbalanceAlgorithmDTO = mapper.readValue(str, ImbalanceAlgorithmDTO.class);
        System.err.println(imbalanceAlgorithmDTO);
    }

    public static String str = "{\n" +
            "  \"code\": 200,\n" +
            "  \"msg\": \"success\",\n" +
            "  \"requestTime\": \"2024-11-23 10:15:44\",\n" +
            "  \"issueTime\": \"2024-11-23 10:15:44\",\n" +
            "  \"data\": {\n" +
            "    \"cycle\": 100,\n" +
            "    \"green_time\": [\n" +
            "      0,\n" +
            "      0,\n" +
            "      72,\n" +
            "      72,\n" +
            "      20,\n" +
            "      20,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0\n" +
            "    ],\n" +
            "    \"cross_id\": \"a5d6b25f8857448584e904690d4b4d2f\",\n" +
            "    \"control_id\": 677903,\n" +
            "    \"provider\": \"hisense\",\n" +
            "    \"plan_no\": \"31\",\n" +
            "    \"offset\": 58,\n" +
            "    \"ring_cnt\": 2,\n" +
            "    \"ring_set\": {\n" +
            "      \"0\": {\n" +
            "        \"0\": [\n" +
            "          5\n" +
            "        ],\n" +
            "        \"1\": [\n" +
            "          4\n" +
            "        ]\n" +
            "      },\n" +
            "      \"1\": {\n" +
            "        \"0\": [\n" +
            "          6\n" +
            "        ],\n" +
            "        \"1\": [\n" +
            "          3\n" +
            "        ]\n" +
            "      }\n" +
            "    },\n" +
            "    \"seq_id\": [\n" +
            "      1,\n" +
            "      2,\n" +
            "      3,\n" +
            "      4,\n" +
            "      5,\n" +
            "      6,\n" +
            "      7,\n" +
            "      8,\n" +
            "      9,\n" +
            "      10,\n" +
            "      11,\n" +
            "      12,\n" +
            "      13,\n" +
            "      14,\n" +
            "      15,\n" +
            "      16\n" +
            "    ],\n" +
            "    \"red_time\": [\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0\n" +
            "    ],\n" +
            "    \"yellow_time\": [\n" +
            "      0,\n" +
            "      0,\n" +
            "      4,\n" +
            "      4,\n" +
            "      4,\n" +
            "      4,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0\n" +
            "    ],\n" +
            "    \"green_flash\": [\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1\n" +
            "    ],\n" +
            "    \"person_flash\": [\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1,\n" +
            "      -1\n" +
            "    ],\n" +
            "    \"min_green\": [\n" +
            "      15,\n" +
            "      14,\n" +
            "      30,\n" +
            "      30,\n" +
            "      14,\n" +
            "      14,\n" +
            "      14,\n" +
            "      14,\n" +
            "      15,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0,\n" +
            "      0\n" +
            "    ],\n" +
            "    \"max_green\": [\n" +
            "      1000,\n" +
            "      1000,\n" +
            "      1000,\n" +
            "      1000,\n" +
            "      1000,\n" +
            "      1000,\n" +
            "      1000,\n" +
            "      1000,\n" +
            "      1000,\n" +
            "      1000,\n" +
            "      1000,\n" +
            "      1000,\n" +
            "      1000,\n" +
            "      1000,\n" +
            "      1000,\n" +
            "      1000\n" +
            "    ],\n" +
            "    \"direction\": [\n" +
            "      [\n" +
            "        [\n" +
            "          0,\n" +
            "          1\n" +
            "        ]\n" +
            "      ],\n" +
            "      [],\n" +
            "      [\n" +
            "        [\n" +
            "          3,\n" +
            "          13\n" +
            "        ],\n" +
            "        [\n" +
            "          0,\n" +
            "          2\n" +
            "        ]\n" +
            "      ],\n" +
            "      [\n" +
            "        [\n" +
            "          1,\n" +
            "          13\n" +
            "        ],\n" +
            "        [\n" +
            "          2,\n" +
            "          2\n" +
            "        ]\n" +
            "      ],\n" +
            "      [\n" +
            "        [\n" +
            "          2,\n" +
            "          13\n" +
            "        ]\n" +
            "      ],\n" +
            "      [\n" +
            "        [\n" +
            "          0,\n" +
            "          13\n" +
            "        ],\n" +
            "        [\n" +
            "          1,\n" +
            "          1\n" +
            "        ]\n" +
            "      ],\n" +
            "      [],\n" +
            "      [],\n" +
            "      [],\n" +
            "      [],\n" +
            "      [],\n" +
            "      [],\n" +
            "      [],\n" +
            "      [],\n" +
            "      [],\n" +
            "      []\n" +
            "    ]\n" +
            "  },\n" +
            "  \"crossId\": \"a5d6b25f8857448584e904690d4b4d2f\",\n" +
            "  \"crossName\": \"龙鼎大道-龙翔路\",\n" +
            "  \"currentAlgo\": 2,\n" +
            "  \"timingPlan\": [\n" +
            "    0,\n" +
            "    0,\n" +
            "    72,\n" +
            "    72,\n" +
            "    20,\n" +
            "    20,\n" +
            "    0,\n" +
            "    0,\n" +
            "    0,\n" +
            "    0,\n" +
            "    0,\n" +
            "    0,\n" +
            "    0,\n" +
            "    0,\n" +
            "    0,\n" +
            "    0\n" +
            "  ]\n" +
            "}";

}
